/* 
* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* (C) Copyright IBM Corp. 2011,2012
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/ 

/*----------------------------------------------------------------------------
File Name:  ODPPCmnAPI.H

Purpose:    Provides the Interface declarations for ODPP, including the 
            function declarations and structure declarations. This file is 
            supposed to be used by the external applications using ODPP. 
            
            Only declarations that are part of the API should be declared 
            here.

Functions:

   Name              Purpose

Structures:

   Name              Purpose

Author:
   Ritesh Tandon     IBM ISL 29/Dec/2009
----------------------------------------------------------------------------*/
#ifndef __ODPPCMNAPIH__
#define __ODPPCMNAPIH__

/*============================================================================
 *                         ODPP VERSION NUMBER, 
 *          THIS SHOULD BE USED FOR ALL STRUCTURES DECLARATIONS
 *==========================================================================*/
#define ODPP_CUR_VERSION               21

#ifndef ODPP_WCHAR
#define ODPP_WCHAR                     wchar_t   
#endif
#define ODPPSVCIDLENGTH                10
#define ODPP_PROVIDER_ID_MAX_LENGTH    ODPPSVCIDLENGTH
#define RETVAL                         int


#define ODPP_MBCS_CHAR_BYTES           6

//Maximum supported buffer size for Provider Name and Description
#define ODPP_PROVIDER_NAME_LEN         50
#define ODPP_PROVIDER_DESCRIPTION_LEN  256

//Maximum supported buffer sizes for Error Message Source,
//Message Test and Message Body
#define ODPP_MAX_ERR_MSG_SRC_LENGTH    (512)   + 10
#define ODPP_MAX_ERR_MSG_TEXT_LENGTH   (512*2) + 50
#define ODPP_MAX_ERR_MSG_BODY_LENGTH   (512*3) + 50

#define ODPP_MAX_PATH_LEN              2048
#define ODPP_MAX_FILENAME_LEN          1024

// List of DBMS Type codes for MBCS/SBCS environments
#define RDB_DB2MVS      '2'         /* DB2 MVS                    */
#define RDB_DB2CS       'C'         /* DB2 Common Server          */
#define RDB_ORACLE      'O'         /* Oracle                     */
#define RDB_SYBASE      'S'         /* Sybase                     */
#define RDB_ODBC        'M'         /* ODBC (syncpoint)           */
#define RDB_INFORMIX    'I'         /* Informix                   */
#define RDB_DB2ZOS      RDB_DB2MVS  /* DB2 zOS (used to be MVS)   */
#define RDB_DB2LUW      RDB_DB2CS   /* DB2 LUW (used to be CS (client server) then UDB (univ. data base) now LUW (Linux Unix Windows))       */
#define RDB_SQLSERVER   'G'         /* MS SQL Server              */
#define RDB_ANY         '*'         /* Used when passing DBMS code to ICU functions to mean "no specific DBMS" or "first match"              */
#define RDB_NONE        'N'         /* Used when passing DBMS code to ICU functions to mean "no DBMS", only use process-specific converters  */
#define RDB_TERADATA    'T'			/* Teradata                     */
#define RDB_NETEZZA     'Z'			/* Netezza*/

//===========================================================================
//                         IMPORTANT NOTE
//===========================================================================
// If specifying CodePage value at the parent structure level for structures 
// DP_SVC_DEF/DP_FRMW_PARAMS_DEF set the code page member of child 
// structures DPFD_MC_SS/DPPRM_VAL_MC_SS/DPPRM_NAME_MC_SS to 
// ODPP_CODE_PAGE_NONE.
// ==========================================================================
// Codepage value set in the child structure(s) always take precedence over 
// codepage value set in parent structure
// ==========================================================================
#define ODPP_CODE_PAGE_NONE            0x80000000

/*============================================================================
 *                         IMPORTANT NOTE
 *============================================================================
 * USE THE BELOW MACRO TO INITIALIZE ODPP STRUCTURES OBJECTS. IT IS IMPORTANT
 * AS THIS MACRO ADD THE VERSION INFO TO THE STRUCTURES
 * 
 *==========================================================================*/
#define EC1(T)                      T ## _EC
#define EYE_CATCHER(TYPE)           EC1(TYPE)

#define INITIALIZE_ODPP_STRUCT(OBJ, TYPE)          \
        {OBJ.sStructVer = ODPP_CUR_VERSION;        \
        memcpy(OBJ.cEyeCatcher,EYE_CATCHER(TYPE),sizeof(OBJ.cEyeCatcher));}

#define INITIALIZE_ODPP_STRUCT_PTR(OBJ_PTR, TYPE)  \
        {OBJ_PTR->sStructVer = ODPP_CUR_VERSION;   \
        memcpy(OBJ_PTR->cEyeCatcher,EYE_CATCHER(TYPE),sizeof(OBJ_PTR->cEyeCatcher));}


/*============================================================================
 *                EYE CATCHER defines
 *==========================================================================*/
/* Eye catcher for DP_ECB */
#define DP_ECB_EC                   "ECBD"

/* Eye catcher for DP_FIELD_DATA_DEF */
#define DP_FIELD_DATA_DEF_EC        "DATA"

/* Eye catcher for DP_ROW_DEF */
#define DP_ROW_DEF_EC               "ROWD"

/* Eye catcher for DP_ROWSET_DEF */
#define DP_ROWSET_DEF_EC            "RWST"

/* Eye catcher for    DP_FIELD_DEF */
#define DP_FIELD_DEF_EC             "FLDA"

/* Eye catcher for DP_INIT_OP_DEF */
#define DP_INIT_OP_DEF_EC           "OPRA"

/* Eye catcher for DP_FRMW_PARAMS_DEF */
#define DP_FRMW_PARAMS_DEF_EC       "FRPA"

/* Eye catcher for DP_SVC_DEF */
#define DP_SVC_DEF_EC               "SVCA"

/* Eye catcher for DP_PRV_DEF */
#define DP_PRV_DEF_EC               "PRVA"

/* Eye catcher for DP_ERR_MSG */
#define DP_ERR_MSG_EC               "EMSG"

/* Eye catcher for DP_PARMGEN_DEF */
#define DP_PARMGEN_DEF_EC           "PGDA"

/* Eye catcher for DP_PARAMS_DEF */
#define DP_PARAMS_DEF_EC            "PRMA"

/* Eye catcher for DP_KVPAIR_DEF */
#define DP_KVPAIR_DEF_EC            "KVPA"

/* Eye catcher for DP_INSTANCE_DEF */
#define DP_INSTANCE_DEF_EC          "INSA"

/*============================================================================
 * Structure declarations to be used by the Optim Data Privacy Provicer API's
 *==========================================================================*/
typedef struct s_odbc_date
{
    signed short     Year;
    unsigned short   Month;
    unsigned short   Day;
} ODPP_ODBC_DATE;

typedef struct s_odbc_time
{
    unsigned short   Hour;
    unsigned short   Minute;
    unsigned short   Second;
} ODPP_ODBC_TIME;

typedef struct s_odbc_timestamp
{
    signed short     Year;
    unsigned short   Month;
    unsigned short   Day;
    unsigned short   Hour;
    unsigned short   Minute;
    unsigned short   Second;
    unsigned int     Fraction;
} ODPP_ODBC_TIMESTAMP;

#define ODPPTOKID(s) ODPP_TOKID_ ## s , 

/* Token Identifiers */
enum DP_ETOKENID {
 ODPP_TOKEN_NODE_UNUSED,
#include "ODPPTokIdsH.h"
 ODPP_NUM_TOKENS
};
#undef ODPPTOKID

//---------------------------------------------------------------------
// Name:          ODPP_TOKEN                                     
// Description:                                                  
// A node in the array within an Error Control Block.            
// It holds an DP_ETOKENID identifier and a pointer to the       
// null-terminated token string                                  
//---------------------------------------------------------------------
typedef struct s_DP_Token
{   
   enum DP_ETOKENID eId;            /* token identifier            */
   ODPP_WCHAR       *pValue;        /* token value string          */
} DP_TOKEN; 

//---------------------------------------------------------------------
// Name:          DP_ECB                                         
// Description:                                                  
// Error Control Block (ECB) holds the error code, 
// the row number where the error occurred 
// and optionally any number of tokens containing specific details                                              
//---------------------------------------------------------------------
typedef struct DP_ECB
{
   struct DP_ECB *pNext;               // Pointer to next ECB        
   char           cEyeCatcher[4];      // Eye catcher
   short          sStructVer;          // Version of this structure
   short          sStructLen;          // Length of this structure    
   short          sNumTokens;          // Number of tokens 
   int            iErrCode;            // Error Code       
   int            iRowNum;             // Input row        
   DP_TOKEN       aToken[1];           // Error tokens                     
} DP_ECB;

//---------------------------------------------------------------------
// Name:          DP_FIELD_DATA_DEF                              
// Description:                                                  
// The Field Data Definition carries the source and target data
//---------------------------------------------------------------------
typedef struct DP_FIELD_DATA_DEF
{
   struct DP_FIELD_DATA_DEF 
                 *pNext;               // Next element in chain of DP_FIELD_DATA_DEF
   char           cEyeCatcher[4];      // eye catcher
   short          sStructVer;          // version of this structure
   short          sStructLen;          // length of this structure    
   char 		      bSrcNull;            // Source Null indicator
   char 		      bDstNull;	         // Dest Null indicator
   int            iSrcBufLen;          // length, or max length of source buffer
                                       // in bytes. On Return in case bCopyToDest of SvcDef 
                                       // is FALSE this variable will be updated to 
                                       // contain the size of the output data in bytes
   int            iDstBufLen;          // length, or max length of dest buffer
                                       // in bytes.  On Return in case bCopyToDest of SvcDef 
                                       // is TRUE this variable will be updated to 
                                       // contain the size of the output data in bytes
   int            iOutBufLen;          // bytes of the data returned in the pDstBuf. In case of 
                                       // ODPPSUCCESS, this is the bytes of return data. If
                                       // RETURN CODE = ODPP_DATACNV_ERR_DATA_TRUNCATED, this 
                                       // is the max bytes required to hold the return data
   int   	      iErrorCode;          // Error code for this data
   unsigned char *pSrcBuf;             // buffer to source buffer
   unsigned char *pDstBuf;             // buffer to destination buffer
} DP_FIELD_DATA_DEF;


//---------------------------------------------------------------------
// Name:          DP_ROW_DEF                                    
// Description:                                                 
// The Row Definition encapsulates the DP_FIELD_DATA_DEF struct   
// to provide a 2-dimensional array of DP_FIELD_DATA_DEF
//                                                              
//********************IMPORTANT****************************************
// bProcessed should be FALSE (i.e. 0) for each row
// in the first Service Call.
//********************IMPORTANT****************************************
//---------------------------------------------------------------------
typedef struct DP_ROW_DEF
{
   struct DP_ROW_DEF   
                 *pNext;               // Next element in chain of DP_ROW_DEF
   char           cEyeCatcher[4];      // eye catcher
   short          sStructVer;          // version of this structure
   short          sStructLen;          // length of this structure    
   char           bProcessed;          // TRUE if the ROW processed earlier
                                       // This should be FALSE (i.e. 0) for each
                                       // row in first Service Call.
   char		      bHasError;	         // TRUE if any column(s) has an error code
   short          sCount;              // Count of DP_FIELD_DATA_DEF(s) either 
                                       // in form of an array or a chain of 
                                       // elements
   DP_FIELD_DATA_DEF   
                 *pFldDataDefine;      // Pointer to an array of 
                                       // DP_FIELD_DATA_DEF(s) OR pointer 
                                       // to the first element of chain of
					                        // DP_FIELD_DATA_DEF.	              
}DP_ROW_DEF;


//---------------------------------------------------------------------
// Name:          DP_ROWSET_DEF                                 
// Description:                                                 
// The Rowset Defintion encapsulates a batch of DP_ROW_DEFs
//---------------------------------------------------------------------
typedef struct DP_ROWSET_DEF
{
   char           cEyeCatcher[4];      // Eye catcher
   short          sStructVer;          // version of this structure
   short          sStructLen;          // length of this structure    
   int            iCount;              // Count of DP_ROW_DEF(s) either in form of 
                                       // an array or a chain of elements
   int            iLastPrcsRow;        // Index of the last row processed. Needed for 
                                       // restart after discard limit failure.
                                       // This should be 0 for first Service Call.
   DP_ROW_DEF   	*pRowDefine;         // Pointer to an array of DP_ROW_DEF(s)
					                        // OR pointer to the first element of chain of
					                        // DP_ROW_DEF.              
}DP_ROWSET_DEF;

//---------------------------------------------------------------------- 
// Name:          DPFD_WC_SS
// Description:                                                  
// The field definition for the wide-character (Unicode)
// sub-structure for DP_FIELD_DEF
//----------------------------------------------------------------------
typedef struct DPFD_WC_SS
{
   ODPP_WCHAR    *pColName;            // Pointer to column name 
                                       // expressed in wide character (Unicode) format
}DPFD_WC_SS;

//---------------------------------------------------------------------- 
// Name:          DPFD_MC_SS
// Description:                                                  
// The field definition for the mixed-character (e.g. SBCS/MBCS)
// sub-structure for DP_FIELD_DEF
//----------------------------------------------------------------------
typedef struct DPFD_MC_SS
{
   char           cColNameDBMSType;    // DBMS type of the column name
   int            iColNameCodePage;    // Code page of the column name
   //Set iColNameCodePage to ODPP_CODE_PAGE_NONE to use code page and DBMSType 
   //supplied by the parent structure DP_SVC_DEF

   char          *pColName;            // Pointer to column name 
                                       // expressed in mixed (SBCS/MBCS) character format
}DPFD_MC_SS;

//---------------------------------------------------------------------- 
// Name:          DP_FIELD_DEF                                   
// Description:                                                  
// The field definition describes all of the attributes of a field      
//----------------------------------------------------------------------
typedef struct DP_FIELD_DEF
{
   struct DP_FIELD_DEF     
                 *pNext;               // Pointer to next element in chain of DP_FIELD_DEF
   char           cEyeCatcher[4];      // Eye catcher
   short          sStructVer;          // Version of this structure 
   short          sStructLen;          // Length of this structure 
   short          sDatatype;           // Data type of the input data 
   int            iLength;             // Column length (for string types)
   int            iPrecision;          // Precision (for decimal types) and ORA_VARNUM
   short          sScale;              // Scale (for decimal data)
   int            iDataCodePage;       // Code page of the data
   char           cDataDBMSType;       // DBMS type of the data
   int            iColNameBytes;       // Size, in bytes, of the column name
   char           cSubType;            // Type of sub-structure
                                       // (W or blank or 0=wide-char, M=mixed (SBCS/MBCS) char
   union COLNAME_SS                    // Union of sub-structures 
   {
      DPFD_WC_SS *pWC;                 // Pointer to wide character (Unicode) sub-structure
      DPFD_MC_SS *pMC;                 // Pointer to mixed character (SBCS/MBCS) sub-structure     
   }CN;

} DP_FIELD_DEF;

//---------------------------------------------------------------------- 
// Name:          DPPRM_NAME_WC_SS
// Description:                                                  
// Wide-character (Unicode) parameter name sub-structure for DP_INIT_OP_DEF
//----------------------------------------------------------------------
typedef struct DPPRM_NAME_WC_SS
{
   ODPP_WCHAR    *pParmName;           // Pointer to parm key
                                       // expressed in wide character (Unicode) format
}DPPRM_NAME_WC_SS;

//---------------------------------------------------------------------- 
// Name:          DPPRM_NAME_MC_SS
// Description:                                                  
// mixed-character (e.g. SBCS/MBCS) parameter name sub-structure for 
// DP_INIT_OP_DEF
//----------------------------------------------------------------------
typedef struct DPPRM_NAME_MC_SS
{
   char           cParamNameDBMSType;  // DBMS type of the parameter name
   int            iParamNameCodePage;  // Code page of the parameter name
   //Set iParamNameCodePage to ODPP_CODE_PAGE_NONE to use code page and DBMSType 
   //supplied by the parent structure (DP_FRMW_PARAMS_DEF/DP_SVC_DEF)

   char          *pParmName;           // Pointer to parm key
                                       // expressed in mixed (SBCS/MBCS) character format
}DPPRM_NAME_MC_SS;

//---------------------------------------------------------------------- 
// Name:          DPPRM_VAL_WC_SS
// Description:                                                  
// Wide-character (Unicode) parameter value sub-structure for DP_INIT_OP_DEF
//----------------------------------------------------------------------
typedef struct DPPRM_VAL_WC_SS
{
   ODPP_WCHAR    *pParamVal;           // Pointer to parm key
                                       // expressed in wide character (Unicode) format
}DPPRM_VAL_WC_SS;

//---------------------------------------------------------------------- 
// Name:          DPPRM_VAL_MC_SS
// Description:                                                  
// mixed-character (e.g. SBCS/MBCS) parameter value sub-structure for 
// DP_INIT_OP_DEF
//----------------------------------------------------------------------
typedef struct DPPRM_VAL_MC_SS
{
   char           cParamValDBMSType;   // DBMS type of the parameter value
   int            iParamValCodePage;   // Code page of the parameter value
   //Set iParamValCodePage to ODPP_CODE_PAGE_NONE to use code page and DBMSType 
   //supplied by the parent structure (DP_FRMW_PARAMS_DEF/DP_SVC_DEF)

   char          *pParamVal;           // Pointer to parm key
                                       // expressed in mixed (SBCS/MBCS) character format
}DPPRM_VAL_MC_SS;

//---------------------------------------------------------------------- 
// Name:          DP_INIT_OP_DEF
// Description:                                                  
// Init Parameter Definition contains all the initialization      
// parameters required for the current service                   
//---------------------------------------------------------------------- 
typedef struct DP_INIT_OP_DEF
{
   struct DP_INIT_OP_DEF
                 *pNext;               //Pointer to next element in chain of parameters
   char           cEyeCatcher[4];      // Eye Catcher
   short          sStructVer;          // Version of this structure 
   short          sStructLen;          // Length of this structure 
   unsigned short usParameterID;       // ODPP Parameter

   char           cParamNameSubType;   // Type of parameter sub-structure
                                       // (W or blank=wide-char, M=mixed (SBCS/MBCS) char
   
   int            iParamNameBufBytes;  // Size (in bytes) of param value buffer

   // =================
   // Parameter name
   // =================
   // Used in scenario when there is no parameter ID 
   // Set usParameterID = 0 when a parameter name is passed instead of ParameterID
   union PARAM_NAME_SS
   {
      DPPRM_NAME_WC_SS
                 *pWC;                 // Pointer to wide character (Unicode) sub-structure 
      DPPRM_NAME_MC_SS
                 *pMC;                 // Pointer to mixed character (SBCS/MBCS) sub-structure 
   }PN;

   // =================
   // Parameter value
   // =================
   int            iValueSubType;       // Type of parameter value as defined below
                                       // any value except these are user defined values
                                       // and indicates the use of pUserVal.
                                       //================================================
                                       // Custom service providers can define their own 
                                       // values for their usage
                                       //================================================
                  #define PARAM_VAL_NONE  0  //This paramaeter has no value
                  #define PARAM_VAL_NUM   1  //Parameter value is numeric
                  #define PARAM_VAL_WC    2  //Parameter value is Wide Char string
                  #define PARAM_VAL_MC    3  //Parameter value is SBCS/MBCS

   int            iValueBufBytes;      // Size (in bytes) of param value buffer

   union PARAM_VAL
   {
      unsigned int 
                  uiVal;               // User-supplied numeric value
      DPPRM_VAL_WC_SS         
                 *pWC;                 // Pointer to wide character (Unicode) sub-structure
      DPPRM_VAL_MC_SS
                 *pMC;                 // Pointer to mixed character (SBCS/MBCS) sub-structure 
      void       *pUserVal;            // User defined parameter value 
   }PV;

}DP_INIT_OP_DEF;


//---------------------------------------------------------------------- 
// Name:          DP_FRMW_PARAMS_DEF
// Description:                                                 
// Encapsulates the parameters required for the 
// ODPP framework initialization
//---------------------------------------------------------------------- 
typedef struct DP_FRMW_PARAMS_DEF
{
   char           cEyeCatcher[4];      // Eye Catcher
   short          sStructVer;          // version of this structure 
   short          sStructLen;          // Length of this structure 
   short          sParamCount;         // Count of parameters in pParams

   // Default Code Page and DBMS Type for Parameter names and values
   //
   // Note: Codepage/DBMSType values set for individual parameters take 
   // precedence over these value
   char           cParamValDBMSType;   // DBMSType of parameter value
   int            iParamValCP;         // Code page of parameter value

   DP_INIT_OP_DEF
                 *pParams;             // List of parameters

}DP_FRMW_PARAMS_DEF;
   
//---------------------------------------------------------------------- 
// Name:          DP_SVC_DEF
// Description:                                                 
// The Service Definition contains the parameters and field definition lists
// and the other service definition type initialization parameters.
//---------------------------------------------------------------------- 
typedef struct DP_SVC_DEF
{
   char           cEyeCatcher[4];      // Eye Catcher
   short          sStructVer;          // version of this structure
   short          sStructLen;          // length of this structure    
   char           bRefreshSVCDef;      // [For Future Use] When TRUE, this 
                                       // causes the existing the existing 
                                       // service definition to be updated
   char		      bCopyToDest;         // Copy the masked data to destination buf
   short	         sParamCount;  	      // Count of elements in pParams
   short	         sFldCount;	         // Count of elements in pFldDef

   // Default Code Page and DBMS Type for Parameter names and values
   //
   // Note: Codepage/DBMSType values set for individual parameters/fields
   // take precedence over these value
   char           cCtrlDataDBMSType;   // DBMSType of control Data
                                       // in DP_INIT_OP_DEF and DP_FIELD_DEF
   int            iCtrlDataCP;         // Code page of control data 
                                       // in DP_INIT_OP_DEF and DP_FIELD_DEF

   DP_INIT_OP_DEF 
                 *pParams;           // Pointer to array or first element in
                                       // chain of DP_INIT_OP_DEF. Provides all 
                                       // needed parameters.
   DP_FIELD_DEF  *pFldDef;             // Pointer to array or first element in
                                       // chain of DP_FIELD_DEF.

}DP_SVC_DEF;


//---------------------------------------------------------------------- 
// Name:          DPPD_WC_SS
// Description:                                                  
// The provider definition for the wide-character (Unicode)
// sub-structure for DP_PRV_DETAILS
//----------------------------------------------------------------------
typedef struct DPPD_WC_SS 
{
   ODPP_WCHAR    *pProviderName;       // Pointer to provider name
                                       // expressed in wide character (Unicode) format
   ODPP_WCHAR    *pDescription;        // Pointer to provider description
                                       // expressed in wide character (Unicode) format
}DPPD_WC_SS;

//---------------------------------------------------------------------- 
// Name:          DPPD_MC_SS
// Description:                                                  
// The provider definition for the mixed-character (e.g. SBCS/MBCS)
// sub-structure for DP_PRV_DETAILS
//----------------------------------------------------------------------
typedef struct DPPD_MC_SS
{
   char          *pProviderName;       // Pointer to provider name
                                       // expressed in mixed (SBCS/MBCS) character format
   char          *pDescription;        // Pointer to provider description
                                       // expressed in mixed (SBCS/MBCS) character format
}DPPD_MC_SS;

//---------------------------------------------------------------------- 
// Name:          DP_PRV_DETAILS
// Description:                                                 
// The Provider Details is an entry in the linked-list of   
// provider-type information that is returned 
// to the caller of Provider_Enumerate
//---------------------------------------------------------------------- 
typedef struct DP_PRV_DETAILS
{
   struct DP_PRV_DETAILS
                 *pNext;		         // Pointer to next element in chain	
   char           bUserDefine;         // When TRUE, this is a user-supplied provider
   char           bLicensed;           // When TRUE, this is a licensed provider
   short          sProviderID;         // ODPP internally defined provider ID
   short          sProviderVer;        // Version of the provider 
   int            iPrvNameBufBytes;    // Size, in bytes, of the provider name
   int            iPrvDescBufBytes;    // Size, in bytes, of the provider description
   union PD_SS
   {
      DPPD_WC_SS *pWC;                 // Pointer to wide character (Unicode) sub-structure
      DPPD_MC_SS *pMC;                 // Pointer to mixed character (SBCS/MBCS) sub-structure
   }PD;
}DP_PRV_DETAILS;

//---------------------------------------------------------------------- 
// Name:          DP_PRV_DEF
// Description:                                                 
// The Provider Definition contians provider details in a linked-list of   
// provider-type information that is returned to the caller of 
// Provider_Enumerate
//---------------------------------------------------------------------- 
typedef struct DP_PRV_DEF
{
   char           cEyeCatcher[4];      // Eye Catcher
   short          sStructVer;          // Version of this structure
   short          sStructLen;          // Length of this structure    

   char           cSubType;            // Type of sub-structure
                                       // (W or blank=wide-char, M=mixed (SBCS/MBCS) char
   char           cPDDBMSType;         // DBMSType of the provider name and description
   int            iPDCodePage;         // Code page of the provider name and description

   DP_PRV_DETAILS
                 *pPrvDetails;         // Array or LinkList of Providers with details
}DP_PRV_DEF;


//---------------------------------------------------------------------- 
// Name:          DPEM_WC_SS
// Description:                                                  
// The Formatted error message definition for the wide-character (Unicode)
// sub-structure for DP_ERR_MSG
//----------------------------------------------------------------------
typedef struct DPEM_WC_SS
{
   int            iMsgSrcBytes;        // Size, in bytes, of the error message source
   int            iMsgBytes;           // Size, in bytes, of the error message description
   int            iMsgBodyBytes;       // Size, in bytes, of the error message body
   ODPP_WCHAR    *pMsgSrc;             // Pointer to error message source
                                       // expressed in wide character (Unicode) format
   ODPP_WCHAR    *pMsg;                // Pointer to error message description
                                       // expressed in wide character (Unicode) format
   ODPP_WCHAR    *pMsgBody;            // Pointer to error message body
                                       // expressed in wide character (Unicode) format
}DPEM_WC_SS;

//---------------------------------------------------------------------- 
// Name:          DPEM_MC_SS
// Description:                                                  
// The provider definition for the mixed-character (e.g. SBCS/MBCS)
// sub-structure for DP_ERR_MSG
//----------------------------------------------------------------------
typedef struct DPEM_MC_SS
{
   char           cEMDBMSType;         // DBMS type of the error message details
   int            iEMCodePage;         // Code Page of the error message details
   int            iMsgSrcBytes;        // Size, in bytes, of the error message source
   int            iMsgBytes;           // Size, in bytes, of the error message description
   int            iMsgBodyBytes;       // Size, in bytes, of the error message body
   char          *pMsgSrc;             // Pointer to error message source
                                       // expressed in mixed (SBCS/MBCS) character format
   char          *pMsg;                // Pointer to error message description
                                       // expressed in mixed (SBCS/MBCS) character format
   char          *pMsgBody;            // Pointer to error message body
                                       // expressed in mixed (SBCS/MBCS) character format
}DPEM_MC_SS;

//---------------------------------------------------------------------- 
// Name:          DP_ERR_MSG
// Description:                                                 
// The Provider Defintioner is an entry in the linked-list of   
// provider-type information that is returned 
// to the caller of Provider_GetFormattedErrorMsg
//---------------------------------------------------------------------- 
typedef struct DP_ERR_MSG
{
   char           cEyeCatcher[4];      // Eye Catcher
   short          sStructVer;          // Version of this structure
   short          sStructLen;          // Length of this structure    
   int            iErrNum;             // Error number/Return code. USED ONLY
                                       // WHEN pECB is NULL, IGNORED OTHERWISE
   int            iLanguage;           // Language to use for error messages
   DP_ECB        *pECB;                // Error Control Block
   char           cSubType;            // Type of sub-structure
                                       // (W or blank=wide-char, M=mixed (SBCS/MBCS) char
   union EM_SS
   {
      DPEM_WC_SS                       // Pointer to wide character (Unicode) sub-structure
                 *pWC; 
      DPEM_MC_SS                       // Pointer to mixed character (SBCS/MBCS) sub-structure
                 *pMC; 
   } EM;
}DP_ERR_MSG;

//---------------------------------------------------------------------- 
// Name:          DP_LIB_LIST
// Description:
// An element in an array of additional ODPP libraries to load.
//---------------------------------------------------------------------- 
typedef struct DP_LIB_LIST
{
   int            iBufBytes;           // Size of the buffer pointed to by pLib in bytes
   char*          pLib;                // Pointer to the name of the shared library to load
}DP_LIB_LIST;

//============================================================================
// API's to be used by applications to use ODPP Services 
//==========================================================================*/

// Dynamically loads the ODPP libraries except the service provider
// libraries and their dependent libraries.
RETVAL Provider_PreLoad(char* pBinPath,
                        int iBinPathBytes,
                        DP_LIB_LIST* pLibList,
                        int iArrCnt);

// This will initialize the framework and all its components
RETVAL Provider_FrmwInit
                           (
                              DP_FRMW_PARAMS_DEF *pInitParams
                           );

// For each Service type that the caller wants to initiate, an Initialization 
// call is required, and a TOKEN is returned to establish a session.  This 
// TOKEN will then be required on all subsequent calls for this session.   
RETVAL Provider_Init
                           (
                              int *pSvcToken, 
                              char *pSvcID, 
                              int iSvcIdLen,
                              DP_SVC_DEF *pSvcdef, 
                              char bAllocInitBlock
                           );

// The appropriate data privacy method would then be requested via a Provider
// Service call to perform data privacy function using pre-determined standard 
// names as described in the Optim Data Privacy Development manual.
// Data privacy method if specific here will overwrite the method specified 
// during Provider_Init using parameter ODPP_OPR_METHOD.
//
// Curently ONLY ODPP_METHOD_MASK OR ODPP_METHOD_DEFAULT IS SUPPORTED FOR sMethod
//
// 3rd Argument, i.e. pSvcDef, is for future use. It should be set to NULL.
//
RETVAL Provider_Service
                           (
                              int iSvcToken, 
                              short sMethod,  
                              DP_SVC_DEF *pSvcDef,       //FOR FUTURE USE
                              DP_ROWSET_DEF *pRowSet
                           );


// When the user is finished with a Service they will be required to make a 
// termination call   
RETVAL Provider_Term
                           (  
                              int iSvcToken
                           );

// Used to list all the available providers along with their details   
RETVAL Provider_Enumerate
                           (
                              DP_PRV_DEF *pPrvDetails, 
                              short *pCount
                           );

 
// Returns the error count and max ECB size 
// This function should not be called while a service call running 
// for the specified service token otherwise behaviour is undefined
RETVAL Provider_GetErrorCount
                           (
                              int iSvcToken,
                              short * pErrCount, 
                              int * pErrAreaLen
                           );

// Returns the list of ECBs starting with the oldest ECB first
// This function should not be called while a service call running 
// for the specified service token otherwise behaviour is undefined
RETVAL Provider_GetError
                           (  
                              int iSvcToken,
                              DP_ECB * pECB
                           );

// Returns the formatted error message
RETVAL Provider_GetFormattedErrorMsg
                           (
                              DP_ERR_MSG *pErrMsg
                           );

// Terminates ECB processing
RETVAL Provider_TermErrors
                           (
                              int iSvcToken
                           );

// Returns the provider specific information requested by 
// sRequest in ptr   
RETVAL Provider_GetInfo
                           (
                              int iSvcToken, 
                              short sRequest,  
                              void *ptr,
                              int *pBufLen
                           );

// This will terminate the framework and all of its components
RETVAL Provider_FrmwTerm
                           (  
                              void  
                           );



//============================================================================
// ODPP PARSER SPECIFIC DEFINES, STRUCTURES
//==========================================================================*/
#define DP_PARSER_ERRMSG_MAX_LEN      512

//---------------------------------------------------------------------- 
// Name:          DP_KVPAIR_DEF
// Description:                                                 
//                Structure to hold key-value pairs returned from ODPP_Parse
//---------------------------------------------------------------------- 
typedef struct DP_KVPAIR_DEF
{
   struct DP_KVPAIR_DEF
                     *pNext;              // Pointer to next element in chain of parameters
   char              cEyeCatcher[4];      // Eye Catcher
   short             sStructVer;          // Version of this structure
   short             sStructLen;          // Length of this structure    
   ODPP_WCHAR		   *pKey;
   unsigned int      uiKeyID;
   int 		         iKeyBytes;           //size in bytes of the allocated pKey buffer
   int 		         iKeyOutBytes;        //bytes written to the pKey buffer include NULL terminator or bytes required if there's an error
   ODPP_WCHAR		   *pValue;             
   unsigned int      uiValueID;
   int 		         iValueBytes;         //size in bytes of the allocated pValue buffer
   int 		         iValueOutBytes;      //bytes written to the pValue buffer include NULL terminator or bytes required if there's an error
   int               iErrorCode;          //Error code if data was truncated
}DP_KVPAIR_DEF;

// Sub-structure contains the DP_PARAMS_DEF parameter string
// in wide-character (UTF-16) format
typedef struct DPPS_WC_SS
{
   ODPP_WCHAR    *pParamString;
                           
}DPPS_WC_SS;

// Sub-structure contains the DP_PARAMS_DEF parameter string
// in mixed-character (SBCS/MBCS) format
typedef struct DPPS_MC_SS
{
   char           cDBMSType;
   int            iCodePage;
   char          *pParamString;
}DPPS_MC_SS;

// Structure contains the Provider_Parse input/output parameters
typedef struct DP_PARAMS_DEF
{
   char                 cEyeCatcher[4];
   short                sStructVer;
   short                sStructLen;
   DP_KVPAIR_DEF        *pKeyValueArr;
   int   	            iKeyValueArrSize;  //number of elements in pKeyValueArr
   int 		            iKeyCount; //[out] number of key-value pairs set into pKeyValueArr after parsing
   int 		            iKeyMaxBytes;
   int 		            iValueMaxBytes;
   char  		         cEnv; //Env code
   char 		            bHasError; //[out] set to TRUE is an error occurred
   unsigned short       usEncoding; //future use
   unsigned short       usConversion; //future use
   char 		            cParamStringType; // (W or 0=wide-char, M=mixed (SBCS/MBCS) char
   int 		            iParamStringBytes; //size in bytes of the data excluding NULL character
   union PARAM_STRING_SS
   {   
      DPPS_WC_SS  *pWC;
      DPPS_MC_SS  *pMC;
   } PS;
}DP_PARAMS_DEF;

typedef struct DP_PARMGEN_DEF
{
   char                 cEyeCatcher[4];
   short                sStructVer;
   short                sStructLen;
   DP_PARAMS_DEF       *pParserParams;
   DP_SVC_DEF          *pSvcDef;
   char                *pSvcId;
   int                  iSvcIdBufLen;
   int                 *pSvcIdLen;
}DP_PARMGEN_DEF;

//List of Env codes
#define ENV_OPTDIS	'D'
#define ENV_UDF		'U'
#define ENV_LUA		'L'
#define ENV_APP	    'A'

//----------------------------------------------------------------------------
//Parser related functions
//----------------------------------------------------------------------------
//Initialise the parser
RETVAL Parser_Init ( void );
//Terminate the parser
RETVAL Parser_Term ( void );

// Call this to convert the ODPP parameter string to ODPP Key-Value pairs 
// (i.e. DP_KVPAIR_DEF) array. If pKeyValueArr is NULL and iKeyValueArrSize 
// value is 0 and the Provider_Parse() method is called the parser returns the 
// number of Key-Value pairs required in pParamDef->iKeyCount, iKeyMaxBytes and 
// iValueMaxBytes have the max key and value bytes required which can be used to 
// allocate pKey and pValue
RETVAL Parser_Service      (
                              DP_PARAMS_DEF *pParamDef
                           );

// Returns the error count and max ECB size 
// This function should not be called while a service call running 
// for the specified service token otherwise behaviour is undefined
RETVAL Parser_GetErrorCount
                           (
                              short * pErrCount, 
                              int * pErrAreaLen
                           );

// Returns the list of ECBs starting with the oldest ECB first
// This function should not be called while a service call running 
// for the specified service token otherwise behaviour is undefined
RETVAL Parser_GetError
                           (  
                              DP_ECB * pECB
                           );

// Returns the formatted error message
RETVAL Parser_GetFormattedErrorMsg
                           (
                              DP_ERR_MSG *pErrMsg
                           );

//----------------------------------------------------------------------------
//Parameter Generator related functions
//----------------------------------------------------------------------------

// Initialize ODPP Parameter generator (Adapter)
RETVAL ParmGen_Init        ( 
                              void                           
                           );

RETVAL ParmGen_Term        ( 
                              void                           
                           );

// Call this to convert the ODPP Key-Value pairs array into a DP_SVC_DEF structure
RETVAL ParmGen_Service     
                           ( 
                              DP_PARMGEN_DEF *pParmGen
                           );

//Free's all the memory allocated for DP_FIELD_DEF structure(s) and 
//DP_INIT_OP_DEF structures
RETVAL ParmGen_Clean       
                           (
                              DP_SVC_DEF *pSvcDef
                           );

// Returns the error count and max ECB size for Parameter Generator
// This function should not be called while a service call running 
// for the specified service token otherwise behaviour is undefined
RETVAL ParmGen_GetErrorCount
                           (
                              short * pErrCount, 
                              int * pErrAreaLen
                           );

// Returns the list of ECBs starting with the oldest ECB first
// This function should not be called while a service call running 
// for the specified service token otherwise behaviour is undefined
RETVAL ParmGen_GetError
                           (  
                              DP_ECB * pECB
                           );

// Returns the formatted error message
RETVAL ParmGen_GetFormattedErrorMsg
                           (
                              DP_ERR_MSG *pErrMsg
                           );

// Structure containing the input to the Instance Manager
typedef struct DP_INSTANCE_DEF
{
   int                  iSvcToken;
   int                  iParmStrBytes;
   int                  iUserDataBytes;
   char                 *pParmStr;
   void                 *pUserData;
}DP_INSTANCE_DEF;

//----------------------------------------------------------------------------
// Platform-specific defines go here
//----------------------------------------------------------------------------

//Function pointer declarations for InstMgr Clean User Data CALLBACK
#if defined(WIN32) || defined(_WIN32)
   typedef void (__stdcall *PINSTMGR_CALLBACK)(void *pUserData);
#else
   typedef void (*PINSTMGR_CALLBACK)(void *pUserData);
#endif

//Initialize the Instance Manager
RETVAL InstMgr_Init(void);

//Lock the Instance Manager mutex
RETVAL InstMgr_Lock(void);

//Searches the list of instances and returns an error if the instance
//corresponding to the parameter string is not found
RETVAL InstMgr_Find(DP_INSTANCE_DEF *pInstanceDef);

//Adds the instance to the internal list of active service provider instances
RETVAL InstMgr_Add(
                     DP_INSTANCE_DEF *pInstanceDef, 
                     PINSTMGR_CALLBACK pCallBack
                   );

//Deletes the instance from the internal list of active service provider instances
RETVAL InstMgr_Delete(DP_INSTANCE_DEF *pInstanceDef);

//Unlock the Instance Manager mutex
RETVAL InstMgr_Unlock(void);

//Terminate the Instance Manager
RETVAL InstMgr_Term(void);

//============================================
//Structures for Provider specific requests
//============================================
//Request for Destination Fields buffer size in bytes: Applicable to only Lookup provider currently
typedef struct DP_REQ_DEST_FLD_INFO
{
   char        cDataDBMSType; // DBMS type of the data
   short       sDataType;     //Data type of the field
   short       sScale;        //Scale (for decimal data)
   int         iIndex;        //Index of the field in the supplied field list
   int         iBytes;        //Maximum Bytes required to return data for this field
   int         iLength;       //Field character length (for string types)
   int         iPrecision;    //Precision (for decimal types) and ORA_VARNUM
   int         iDataCodePage; //Code page of the data
   int         iColNameBytes; //Size, in bytes, of the column name
   ODPP_WCHAR *pNameW;        //Name of the field. [This field need not be allocated 
                              //or freed as it is mananged by ODPP internally]
}DP_REQ_DEST_FLD_INFO;

#endif //__ODPPCMNAPIH__
 
